#
# Cooking skills.
#
init python:

    cookMasteriesAtC = ("a beginner", "competent", "good", "an expert", "a master")
    cookMasteriesAdjC = ("not great", "okay", "fair", "good", "very good", "really good", "perfect")

    #
    # Info for each recipe
    #
    class CookRecipe:
        experienceM = 0                     # Experience
        foodKeyM = None                     # Key to use in food dict
        foodNameM = None                    # Human name of recipe
        servingsM = 4                       # Maximum that can be made in one go.
        tagsM = None                        # Tags for this recipe

        # Constructor.
        #
        # foodKey   - key into food dict
        # foodName  - human name of recipe
        # tags      - tags for this recipe
        # servings  - maximum servings made in one go
        # exp       - experience with cooking this 1..100
        #
        def __init__(self, foodKey, foodName, tags, servings=4, exp=1):
            self.experienceM = exp
            self.foodKeyM = foodKey
            self.foodNameM = foodName
            self.servingsM = servings
            self.tagsM = tags


    #
    # Cooking skills.
    #
    class CookSkill:
        costFacM = 0.5                      # Cost factor for home cooking
        debugStrM = "Empty"                 # Debug string
        dishesM = dict()                    # Known dishes
        quantM = 5                          # Number of expertise levels

        # Constructor.
        #
        # dishDict  - dictionary for known dishes, should be defaulted RenPy RevertableDict.
        # costFac   - cost factor when home cooking
        # quant     - number of expertise levels
        #
        def __init__(self, dishDict, costFac=0.5, quant=5):
            self.costFacM = costFac
            self.quantM = quant
            self.dishesM = dishDict
            self.debugStrM = "Empty"

        # ---------------------------------------------------------------------
        # Accessors
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Add experience with an existing recipe.
        # Limits experience to 0..100.
        # If the recipe is not known do nothing.
        #
        # recipe    - recipe key
        # delta     - experience to add
        # Returns True if the new mastery is different to the old.
        #
        def addExp(self, recipe, delta):
            result = False
            if recipe in self.dishesM:
                exp = self.dishesM[recipe].experienceM
                oldQuant = self.quant(exp)
                exp = max(0, min(100, exp + delta))
                newQuant = self.quant(exp)
                result = newQuant != oldQuant
                self.dishesM[recipe].experienceM = exp
                self.dbgStrM = self.debug()
                if debug:
                    dbg("CookSkill.addExp({}, {})={} exp={:0.1f}".format(recipe, delta, result, exp))
            else:
                dbg("CookSkill.addExp({}, {}): unknown recipe".format(recipe, delta))
            return result

        # Add a new recipe.
        #
        # recipe    - recipe key
        # foodKey   - food key (see FoodDict)
        # foodName  - human name
        # servings  - servings made
        # exp       - base experience 1..100
        #
        def addRecipe(self, recipe, foodKey, foodName, servings, tags, exp=1):
            recipeEntry = CookRecipe(foodKey, foodName, tags, servings, exp)
            self.dishesM[recipe] = recipeEntry
            self.dbgStrM = self.debug()
            if debug:
                dbg("CookSKill:.addRecipe('{}', '{}', '{}', {}, '{}', {})".format(recipe, foodKey, foodName, servings, tags, exp))

        # Add a new recipe from the dictionary.
        #
        # recipe    - recipe key
        # exp       - base experience 1..100
        #
        def addRecipeDict(self, recipe, exp=1):
            if recipe in recipesC:
                tuple = recipesC[recipe]
                foodKey = tuple[0]
                servings = tuple[1]
                foodName = tuple[2]
                tags = tuple[4]
                if debug:
                    dbg("CookSkill.addRecipeDict('{}', {})".format(recipe, exp))
                self.addRecipe(recipe, foodKey, foodName, servings, tags, exp)
            else:
                dbg("CookSkill.addRecipeDict('{}', {}) unknown recipe".format(recipe, exp))

        # Get cost factor for home-made food.
        #
        def getCostFac(self):
            return self.costFacM

        # Get the current experience value with a recipe 0..100.
        # If the recipe is not known return 0
        #
        # recipe    - recipe key
        #
        def getExp(self, recipe):
            if recipe in self.dishesM:
                return self.dishesM[recipe].experienceM
            return 0

        # Get the food key of a recipe.
        # If the recipe is not known return None
        #
        # recipe    - recipe key
        #
        def getFoodKey(self, recipe):
            if recipe in self.dishesM:
                return self.dishesM[recipe].foodKeyM
            return None

        # Get the food name of a recipe.
        # If the recipe is not known return None
        #
        # recipe    - recipe key
        #
        def getFoodName(self, recipe):
            if recipe in self.dishesM:
                return self.dishesM[recipe].foodNameM
            return None

        # Get the label for an activity.
        # Uses cook{recipe}.{activity}{quant(exp)} if it exists,
        # or cook{recipe}.{activity} as a fallback.
        # Returns None if neither exist or the recipe is not known.
        #
        # recipe    - the recipe key to use
        # activity  - the activity such as 'cook' or 'eat'
        #
        def getLabel(self, recipe, activity):
            result = None
            if recipe in self.dishesM:
                quant = self.quant(self.dishesM[recipe].experienceM)
                guess = "cook{}.{}{}".format(recipe.capitalize(), activity, quant)
                if debug:
                    dbg("CookSkill.getLabel('{}', '{}') trying '{}'".format(recipe, activity, guess))
                if world.hasLabel(guess):
                    result = guess
                else:
                    guess = "cook{}.{}".format(recipe.capitalize(), activity)
                    if debugM:
                        dbg("CookSkill.getLabel('{}', '{}') trying '{}'".format(recipe, activity, guess))
                    if world.hasLabel(guess):
                        result = guess
                if result is None:
                    dbg("CookSkill.getLabel('{}', '{}'): no matching label '{}'".format(recipe, activity, guess))
                elif debug:
                    dbg("CookSkill.getLabel('{}', '{}')='{}'".format(recipe, activity, result))
            else:
                dbg("CookSkill.getLabel('{}', '{}'): unknown recipe".format(recipe, activity))
            return result

        # Get the mastery of a recipe.
        # Result is an integer between 0 and quant.
        #
        # recipe    - recipe key
        # buckets   - number of quantisation buckets
        #
        def getMastery(self, recipe, buckets=None):
            if recipe in self.dishesM:
                return self.quant(self.dishesM[recipe].experienceM, buckets)
            return 0

        # Get the mastery adjective string for a recipe.
        #
        # recipe    - recipe key
        #
        def getMasteryAdjStr(self, recipe):
            return cookMasteriesAdjC[self.getMastery(recipe, len(cookMasteriesAdjC)-1)]

        # Get the "mastery at" string for a recipe.
        #
        # recipe    - recipe key
        #
        def getMasteryAtStr(self, recipe):
            return cookMasteriesAtC[self.getMastery(recipe, len(cookMasteriesAtC)-1)]

        # Get the serving size of a recipe.
        # If the recipe is not known return None
        #
        # recipe    - recipe key
        #
        def getServings(self, recipe):
            if recipe in self.dishesM:
                return self.dishesM[recipe].servingsM
            return 0

        # Get tags for a known recipe.
        # Result may be None or a String.
        #
        # recipe    - recipe key
        #
        def getTags(self, recipe):
            if recipe in self.dishesM:
                return self.dishesM[recipe].tagsM
            return None

        # Get a list of recipes that must be learnt before a recipe can be attempted.
        # Returns a list of recipes, empty if all prereqs are satisfied.
        # Returns None if recipe is not known.
        #
        # recipe    - recipe key
        #
        def getUnknowns(self, recipe):
            result = None
            if recipe in recipesC:
                tuple = recipesC[recipe]
                prereqs = tuple[3]
                result = []
                if prereqs is not None:
                    for prereq in prereqs.split():
                        if prereq in self.dishesM:
                            if self.getExp(recipe) == 0:
                                result.append(prereq)
                        else:
                            result.append(prereq)
                if debug:
                    dbg("CookSkill.getUnknowns('{}')={}".format(recipe, result))
            else:
                dbg("CookSkill.getUnknowns('{}') unknown recipe".format(recipe, result))
            return result

        # Get a list of human names of recipes that must be learnt before a
        # recipe can be attempted.
        # Returns a list of names, empty if all prereqs are satisfied.
        # Returns None if recipe is not known.
        #
        # recipe    - recipe key
        #
        def getUnknownNames(self, recipe):
            result = None
            unknowns = self.getUnknowns(recipe)
            if unknowns is not None:
                result = []
                for unknown in unknowns:
                    prereq = recipes[unknown]
                    result.append(prereq[2])
                return result

        # Is skill with a recipe greater than or equal to a threshold?
        #
        # recipe    - recipe key
        # thresh    - threshold to test, or None (default) for any experience
        #
        def isExpGte(self, recipe, thresh=None):
            exp = self.getExp(recipe)
            if thresh is None:
                return exp > 0
            return exp >= thresh

        # Is a recipe known?
        #
        # recipe    - recipe key
        #
        def isKnown(self, recipe):
            return self.getExp(recipe) > 0

        # Can a recipe be learnt?
        #
        # recipe    - recipe key
        #
        def isLearnable(self, recipe):
            if recipe in recipesC:
                unknowns = self.getUnknowns(recipe)
                result = unknowns is None or len(unknowns) == 0
            else:
                dbg("CookSkill.isLearnable({}) unknown recipe".format(recipe))
                result = False
            return result

        # Is a recipe mastered?
        #
        # recipe    - recipe key
        #
        def isMastered(self, recipe):
            return self.getExp(recipe) >= 100

        # Set experience with an existing recipe.
        # Limits experience to 0..100.
        # If the recipe is not known do nothing.
        #
        # recipe    - recipe key
        # exp       - new experience value 0..100
        #
        def setExp(self, recipe, exp):
            if recipe in self.dishesM:
                result = max(0, min(100, exp))
                self.dishesM[recipe].experienceM = result
                self.dbgStrM = self.debug()

        # Set the map to use.
        # Map should be defaulted RenPy RevertableDict.
        #
        # cookMap   - new map
        #
        def setMap(self, cookMap):
            self.dishesM = cookMap



        # ---------------------------------------------------------------------
        # Internals
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # List the recipes and experience.
        #
        def debug(self, group=4):
            total = len(self.dishesM)
            if total == 0:
                return "Empty"

            count = 1
            result = ""
            for name in sorted(self.dishesM.keys()):
                result += "{}={:0.1f}".format(name, self.dishesM[name].experienceM)
                total -= 1
                if count < group:
                    result += " "
                    count += 1
                elif total > 0:
                    result += "\n"
                    count = 0
            return result

        # Quantize an exp value.
        #
        # exp   - experience value
        # buckets   - number of quantization buckets, None (default) for default value
        #
        def quant(self, exp, buckets=None):
            if buckets is None:
                buckets = self.quantM
            return int(exp) * buckets // 100

# Recipe library.
#
# recipeKey -> (foodKey, servings, foodName, prereqs, tags)
#
define recipesC = {
    'burger'    : ('burger',    4, "burgers",           None,       "sav usa"),
    'burgc'     : ('burgc',     4, "cheeseburgers",     'burger',   "sav usa"),
    'frybfast'  : ('frybfast',  4, "fried breakfast",   None,       "sav usa bfast"),
}
