# Game date and time tracking class.
#
init python:

    # Names of days.
    #
    dayNames = [
        'Monday',
        'Tuesday',
        'Wednesday',
        'Thursday',
        'Friday',
        'Saturday',
        'Sunday'
    ]

    #
    # Details of a scheduled event.
    #
    class SchedEvent:
        dayNum = -1             # Day number event is scheduled
        startHr = 0             # Start time of event
        endHr = 0               # End time of event
        evLabel = None          # Label to call when event runs
        rejLabel = None         # Label to call if event rejected
        desc = None             # Description if pc has a choice about the event



        # Is this event current?
        #
        def check(self, dayNum, hour):
            if self.dayNum != dayNum:
                return False
            if hour < self.startHr:
                return False
            if self.endHr != 0 and hour > self.endHr:
                return False

            return True

        # Is this event expired?
        #
        def isExpired(self, dayNum, hour=0):
            if dayNum > self.dayNum:
                result = True
            elif self.endHr != 0 and hour > self.endHr:
                result = True
            else:
                result = False
            if debugM:
                dbg("SchedEvent.isExpired({}, {})={} on {}".format(dayNum, hour, result, self))
            return result

        # String representation for debug.
        #
        def __str__(self):
            return "on {},{:02}:00-{:02}:00 do {}".format(self.dayNum, self.startHr, self.endHr, self.evLabel)

    #
    # Game date and time.
    #
    class GameTime:
        dayName = "Monday"      # Day of week string
        dayAbbr = "Mon"         # Day of week abbreviated
        dayNum = 1              # Day number in game
        dayOw = 0               # Day of week (0:Mon, 1:Tue, ...)
        hour = 0                # Hour of day
        hourStr = "00:00"       # Hour of day string (24hr clock)
        evList = []             # Event list
        evStr = "Unused"        # Event debug string

        # Constructor.
        #
        # evList    - event list
        # dayOw     - day of week (0:Mon, 1:Tue, ...), default 0
        # dayNum    - day number, default 1
        # hour      - hour of day 0..23, default 0
        #
        def __init__(self, evList, dayOw=0, dayNum=1, hour=0):
            self.evList = evList                        # Pending event list
            self.evStr = "Unused"                       # Event debug string
            self.dayOw = dayOw                          # Day of week (0:Mon, 1:Tue, ...)
            self.dayName = self.getDayName()            # Full day name
            self.dayAbbr = self.getDayNameAbbr()        # Abbreviated day name
            self.dayNum = dayNum                        # Day number in game
            self.hour = hour                            # Hour of day 0..23
            self.hourStr = "{:02}:00".format(self.hour) # Hour of day string (24hr clock)

        # ---------------------------------------------------------------------
        # Methods
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Advance time by a number of hours.
        # Roll the day over if this goes beyond midnight, but
        # don't clear day flags, day counters, or expire events.
        #
        def addHours(self, num):
            self.hour += num

            if self.hour > 23:
                self.hour -= 24
                self.dayNum += 1
                self.dayOw += 1

                if self.dayOw > 6:
                    self.dayOw -= 7

                self.dayName = self.getDayName()
                self.dayAbbr = self.getDayNameAbbr()

            self.hourStr = "{:02}:00".format(self.hour)
            if isSelfVoicing():
                # Announce any time change.
                #
                alt("It is now {}.".format(clockTime(self.hour)))


        # Advance time to a specific hour.
        # If the hour is already correct, do nothing.
        # If rollDay is true (default) allow advance to next day when needed,
        # If rollDay is false and hour is less than current time do nothing.
        # Return the number of hours added if positive, otherwise zero.
        #
        def advanceTo(self, hour, rollDay=True):
            delta = hour - self.hour
            if rollDay and delta < 0:
                delta += 24
            if delta > 0:
                self.addHours(delta)
            else:
                delta = 0
            if debugM:
                dbg("GameTime.advanceTo({}, {})={}".format(hour, rollDay, delta))
            return delta

        # Advance to next day and given hour.
        # If the current hour is less than or equal to the requested one
        # just change the hour, not the day.
        #
        def nextDay(self, hour):
            if self.hour > hour:
                self.dayNum += 1
                self.dayOw += 1

                if self.dayOw > 6:
                    self.dayOw -= 7

                self.dayName = self.getDayName()
                self.dayAbbr = self.getDayNameAbbr()
                self.expireEvents(self.dayNum)

            self.setHour(hour)
            dayFlags.clearAll()
            dayCounters.clearAll()
            cooldowns.decAllRm()

        # Get relative time of a day as a string.
        # "today", "tomorrow", "on Wednesday", "next week", "a week on Monday"
        #
        def deltaDayStr(self, evDayNum):
            delta = evDayNum - self.dayNum
            if delta == 0:
                return "today"
            if delta == 1:
                return "tomorrow"
            if delta == 7:
                return "next week"
            weeks = delta / 7
            delta += self.dayOw
            if weeks == 0:
                return "on " + dayNames[delta % 7]
            if weeks == 1:
                return "a week on " + dayNames[delta % 7]
            return cardinal(weeks) + " weeks on " + dayNames[delta % 7]

        # Get the current day number.
        #
        def getDay(self):
            return self.dayNum

        # Get the day of the week as a string.
        #
        def getDayName(self):
            return dayNames[self.dayOw]

        # Get the day of the week as a string.
        #
        def getDayNameAbbr(self):
            switcher={
                0:'Mon',
                1:'Tue',
                2:'Wed',
                3:'Thu',
                4:'Fri',
                5:'Sat',
                6:'Sun'
            }
            return switcher.get(self.dayOw, "")

        # Get the next day number for a given day of week.
        # If allowToday is false (default) this can never be today.
        #
        def getNextDayNum(self, dayOw, allowToday=False):
            day = (dayOw - self.dayOw) % 7
            if not allowToday and day == 0:
                day = 7
            day += self.dayNum
            if debugM:
                dbg("GameTime.getNextDayNum({},{})={}".format(dayOw, allowToday, day))
            return day

        # Get the fractional part of a year elapsed.
        #
        def getYearFrac(self):
            return self.dayNum /365.25

        # Is the time after the given hour (inclusive)?
        #
        def isAfter(self, hour):
            return self.hour >= hour

        # Is the time before the given hour (exclusive)?
        #
        def isBefore(self, hour):
            return self.hour < hour

        # Is it daytime? 6am - 6pm
        #
        def isDaytime(self):
            return self.hour >=6 and self.hour <= 18

        # Is it the weekend?
        #
        def isWeekend(self):
            return self.dayOw >= 5

        # Is the time between two hours?
        #
        def isBetween(self, start, end=25):
            return self.hour >= start and self.hour < end

        # Set the day number.
        # Adjusts day of week and day name.
        #
        def setDay(self, dayNum):
            delta = dayNum - self.dayNum
            self.dayNum = dayNum
            self.dayOw += delta
            self.dayOw %= 7
            self.dayName = self.getDayName()
            self.dayAbbr = self.getDayNameAbbr()
            if debug:
                dbg("GameTime.setDay({})={}".format(dayNum, self.dayName))

        # Set the day of the week (0=Monday,...)
        #
        def setDayOfWeek(self, dayOw):
            self.dayOw = dayOw
            self.dayName = self.getDayName()
            self.dayAbbr = self.getDayNameAbbr()

        # Set the list to use for events.
        #
        def setEventList(self, list):
            self.evList = list
            if debug:
                dbg("GameTime.setEventList({})".format(type(list)))

        # Set the hour of the day (0..23)
        #
        def setHour(self, hour):
            self.hour = hour
            self.hourStr = "{:02}:00".format(hour)

        # ---------------------------------------------------------------------
        # Event methods
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Clear all pending events.
        #
        def clearEvents(self):
            self.evList.clear()
            self.evStr = self.debugEvents()

        # Get any event due now.
        # Returns SchedEvent or None.
        #
        def getEventNow(self):
            return self.getEvent(self.dayNum, self.hour)

        # Get any event due on the given day and time.
        # Returns SchedEvent or None.
        #
        def getEvent(self, dayNum, hour):
            for sched in self.evList:
                if sched.check(dayNum, hour):
                    return sched
            return None

        # Get an event with by it's label.
        # Returns SchedEvent or None.
        #
        def getEventByLabel(self, evLabel):
            for sched in self.evList:
                if sched.evLabel == evLabel:
                    return sched
            return None

        # Is there a pending event to invoke a given label?
        #
        def existsEventFor(self, evLabel):
            for sched in self.evList:
                if sched.evLabel == evLabel:
                    return True
            return False

        # Remove a completed event.
        #
        def doneEvent(self, sched):
            if sched in self.evList:
                self.evList.remove(sched)
                self.evStr = self.debugEvents()

        # How many events are there?
        # Workaround for odd behaviour of len(self.evList)
        #
        def lenEvents(self):
            count = 0
            for sched in self.evList:
                count += 1
            return count


        # Schedule an event.
        # If the day of week is today's day, schedule a week later.
        #
        # evLabel   renpy label to call
        # dayOw     day of week 0:Monday 1:Tuesday ...
        # startHr   optional start hour (default start of day)
        # endHr     optional end hour (default end of day)
        #
        # Returns the SchedEvent or None (if there is a conflicting event)
        #
        def schedule(self, evLabel, dayOw, startHr=0, endHr=0):
            day = self.getNextDayNum(dayOw)
            return self.scheduleOnDay(evLabel, day, startHr, endHr)

        # Schedule an event on the next free day.
        #
        def scheduleNext(self, evLabel, dayNum, startHr=0, endHr=0):
            sched = None
            while sched == None:
                sched = self.scheduleOnDay(evLabel, dayNum, startHr, endHr)
                if sched == None:
                    dayNum += 1
            return sched

        # Schedule an event on a day.
        #
        # evLabel   renpy label to call
        # dayNum    day number
        # startHr   optional start hour (default start of day)
        # endHr     optional end hour (default end of day)
        #
        # Returns the SchedEvent or None (if there is a conflicting event)
        #
        def scheduleOnDay(self, evLabel, dayNum, startHr=0, endHr=0):
            sched = None
            if self.getEvent(dayNum, startHr) == None:
                sched = SchedEvent()
                sched.dayNum = dayNum
                sched.startHr = startHr
                sched.endHr = endHr
                sched.evLabel = evLabel
                self.evList.append(sched)
                self.evStr = self.debugEvents()
            if debug:
                dbg("GameTime.scheduleOnDay({}, {}, {}, {})={}".format(evLabel, dayNum, startHr, endHr, sched))
            return sched

        # Remove events that have expired on the given day.
        #
        def expireEvents(self, dayNum):
            # dbg("GameTime.expireEvents({})".format(dayNum))
            discard = []
            for sched in self.evList:
                # dbg(str(sched))
                if sched.isExpired(dayNum):
                    discard.append(sched)
            for sched in discard:
                self.evList.remove(sched)
                if debug:
                    dbg("GameTime.expireEvents({}) removed {}".format(dayNum, sched))
            self.evStr = self.debugEvents()

        # Remove events with a given label.
        #
        # evLabel   - the label on events to remove
        #
        def removeEvents(self, evLabel):
            discard = []
            for sched in self.evList:
                if sched.evLabel == evLabel:
                    discard.append(sched)
            for sched in discard:
                self.evList.remove(sched)
            self.evStr = self.debugEvents()

        # Remove events with a like a label.
        #
        # evLabel   - the label on events to remove
        #
        def removeEventsWith(self, part):
            discard = []
            chkStr = part.lower()
            for sched in self.evList:
                if chkStr in sched.evLabel.lower:
                    discard.append(sched)
            for sched in discard:
                self.evList.remove(sched)
            self.evStr = self.debugEvents()

        # List the scheduled events (for debug).
        #
        def debugEvents(self, group=2):
            count = 1
            result = ""
            for sched in self.evList:
                result += str(sched)
                if count < group:
                    result += " "
                    count += 1
                else:
                    result += "\n"
                    count = 0

            # Work around for len() failing on rollback
            if result == "":
                result = "Empty"

            return result


# EoF
