#
# Simulation of fans and earnings.
#
init python:

    # Class representing a fan base.
    # Progress follows a sine curve.
    #
    class FanBase:
        activeM = False             # Active or not
        progressM = 0               # Progress 0 .. 100
        maxFansM = 1000             # Maximum number of fans possible
        maxIncomeM = 1000           # Maximum income possible per period
        maxVideosM = 100            # Maximum videos that count towards natural growrh
        accM = 0                    # Accumulated earnings (float)
        lastFansM = 0               # Last number of fans
        lastIncomeM = 0             # Last periods income
        lastVideoM = None           # Name/label of last video
        videosM = 0                 # Videos recorded
        vidTodayM = False           # New video today
        vidDiffTodayM = False       # New kind of video today

        # Advance progress.
        #
        def addProg(self, adj):
            self.progressM += adj
            if self.progressM < 0:
                self.progressM = 0
            elif self.progressM > 100:
                self.progressM = 100
            if debug:
                dbg("addProg({}) = {:0.2f}".format(adj, self.progressM))

        # Upload a video.
        #
        # Increments videos. Sets vidToday which prevents decay.
        # If vidLabel is None or different to last video, set vidDiffToday, and
        # return True. Otherwise return False
        #
        # vidLabel  - label/name of the video
        #
        def addVideo(self, vidLabel=None):
            result = False
            self.videosM += 1
            self.vidTodayM = True
            if vidLabel == None:
                self.vidDiffTodayM = True
                result=True
            elif vidLabel != self.lastVideoM:
                self.lastVideoM = vidLabel
                self.vidDiffTodayM = True
                result=True
            if debug:
                dbg("addVideo({})={}".format(vidLabel, result))
            return result

        # Earn money directly.
        #
        def earn(self, money):
            self.accM += money

        # Natural growth of channel.
        # At a minimum adds 0.04 to progress (complete in 2,500 periods!)
        # At maximum adds 0.32 to progress (complete in 312 days)
        # However there is a natural decay of 0.5% unless a video was uploaded.
        #
        def growth(self):
            if not self.activeM:
                return
            # Use sin of progress so it starts weak, builds, then tails off.
            fanFac = max(0.1, math.sin(math.pi * self.progressM / 100))
            vidFac = min(1, float(self.videosM) / self.maxVideosM)
            self.vidDiffTodayM = False
            if self.vidTodayM:
                decFac = 0
                self.vidTodayM = False
            else:
                decFac = self.progressM / 200.0;

            # Calc adjustment 0.04 to 0.32
            padj = math.log10(1.1 + fanFac * vidFac)
            adj = padj - decFac
            if debug:
                dbg("growth()={:0.5f}\n f={:0.5f} v={:0.4f} m={:0.5f} p={:0.5f} d={:0.5f}".format(adj, fanFac, vidFac, fanFac * vidFac, padj, decFac))
            self.addProg(adj)


        # Get the current number of fans.
        #
        def fans(self):
            if not self.activeM:
                return 0
            return int(math.ceil(self.maxFansM * self.factor()))

        # Get the current income (per day).
        # Result is a float.
        #
        def income(self):
            if not self.activeM:
                return 0;
            return self.maxIncomeM * self.factor()

        # Price a custom video.
        #
        def priceCustom(self, fac, minEarn=5.0):
            if not self.activeM:
                return 0.0
            calc = self.income() * fac
            earn = max(minEarn, calc)

            if earn < 10:
                earn = floorPrec(earn, 0.25)
            elif earn < 20:
                earn = floorPrec(earn, 1)
            elif earn < 100:
                earn = roundPrec(earn, 5)
            elif earn < 300:
                earn = roundPrec(earn, 10)
            else:
                earn = roundPrec(earn, 25)
            if debug:
                dbg("priceCustom({}, {})={:0.2f} calc={}".format(fac, minEarn, earn, calc))
            return earn

        # Withdraw income from the balance.
        #
        def withdraw(self, money):
            if money > 0:
                money = min(math.floor(self.accM * 4.0) / 4.0, money)
            self.accM -= money
            return money

        # Get the balance.
        # Rounded down to the nearest quarter.
        #
        def getBalance(self):
            return math.floor(self.accM * 4.0) / 4.0

        # Get the number of fans at max progress.
        #
        def getFansMax(self):
            return self.maxFansM

        # Get the income at max progress.
        #
        def getIncomeMax(self):
            return self.maxIncomeM

        # Get income over the last period.
        #
        def getLastIncome(self):
            return self.lastIncomeM

        # Get the label/name of the last video uploaded. May be None.
        #
        def getLastVideo(self):
            return self.lastVideoM

        # Get the max number of videos that count towards income.
        #
        def getVideosMax(self):
            return self.maxVideosM

        # Get the number of new fans since this was last called.
        #
        def getNewFans(self):
            if not self.activeM:
                return 0
            current = self.fans()
            delta = current - self.lastFansM;
            self.lastFansM = current;
            return delta

        # Get the periods income.
        # Transfers it to balance.
        # Sets lastIncomeM to nearest quarter and returns this.
        #
        def getNewIncome(self):
            if not self.activeM:
                return 0.0
            oldAcc = math.floor(self.accM * 4.0) / 4.0
            self.accM += self.income()
            newAcc = math.floor(self.accM * 4.0) / 4.0
            self.lastIncomeM = newAcc - oldAcc
            if debug:
                dbg("getNewIncome()={:0.2f} inc={:0.3f} bal={:0.2f}".format(self.lastIncomeM, self.income(), self.getBalance()))
            return self.lastIncomeM

        # Get the progress. 0..100
        #
        def getProgress(self):
            return self.progressM

        # Get the total number of vidoes uploaded
        #
        def getVideos(self):
            return self.videosM

        # Is this fan base active?
        #
        def isActive(self):
            return self.activeM

        # Is progress greater or equal to minval?
        #
        def isProgress(self, minval):
            return self.progressM >= minval

        # Do I have new fans?
        #
        def isNewFans(self):
            if not self.activeM:
                return False
            return self.fans() > self.lastFansM

        # Have I have new fans?
        #
        def isLostFans(self):
            if not self.activeM:
                return False
            return self.fans() < self.lastFansM

        # Have I uploaded a different kind of video today?
        #
        def isVidDiffToday(self):
            return self.vidDiffTodayM

        # Have I uploaded any video today?
        #
        def isVidToday(self):
            return self.vidTodayM

        # Set the fan base active.
        #
        def setActive(self, active):
            self.activeM = active

        # Set progress to have a given number of fans
        #
        def setFans(self, fans):
            # self.progressM = math.acos(float(fans) / self.maxFansM - 1.0) * 100 / math.pi
            self.progressM = 100 - math.acos(float(fans) / self.maxFansM - 1.0) * 100 / math.pi
            if debug:
                dbg("setFans({}) progress={:0.5f}".format(fans, self.progressM))

        # Set the maximum number of fans possible.
        #
        def setFansMax(self, fans):
            self.maxFansM = fans

        # Set the maximum income possible.
        #
        def setIncomeMax(self, money):
            self.maxIncomeM = money

        # Set the maximum videos that contribute towards growth.
        #
        def setVideosMax(self, vids):
            self.maxVideosM = vids

        # Compute the factor of progress.
        # Result is in the range 0 .. 1.0
        #
        def factor(self):
            return (1.0 - math.cos(math.pi * self.progressM / 100)) / 2.0
