# An item on a menu
#
init python:

    # Replacement for a food menu item.
    #
    class FoodItem(object):

        def __init__(self, prompt, mealSize=0, treat=False):
            """
            Construct a food item on a menu.

            prompt      - the text for the item
            mealSize    - the volume of the item, default 0
            treat       - is the item free, default False
            """
            self._promptM = prompt          # Replacement text
            self._mealSizeM = mealSize      # Meal size in small plates
            self._treatM = treat            # Is this "free"?
            self._costM = 0                 # Cost of item
            self._headerM = False           # Is this a section header?
            self._hiddenM = False           # Is the item hidden?
            self._tooBigM = False           # Is this too big to consume?
            self._tooGreedyM = False         # Would it be too greedy to consume?
            self._tooSmallM = False         # Is this too small to bother consuming?
            self._tooPriceyM = False        # Is this too expensive?
            self._postM = ""                # Additional annotation.



        # ---------------------------------------------------------------------
        # Properties
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        @property
        def available(self):
            """
            Can this item be selected?
            """
            return not (self._headerM or self._hiddenM or self._tooBigM or self._tooGreedyM or self._tooSmallM or self._tooPriceyM)

        @property
        def cost(self):
            """
            Get the cost of the item in dollars.
            """
            return self._costM

        @cost.setter
        def cost(self, dollars):
            """
            Set the cost of the item in dollars.
            """
            self._costM = dollars

        @property
        def header(self):
            """
            Is this a header?
            """
            return self._headerM

        @header.setter
        def header(self, boolValue):
            """
            Control whether this item is a header.
            """
            self._headerM = boolValue

        @property
        def hidden(self):
            """
            Is this item hidden in the menu?
            """
            return self._hiddenM

        @hidden.setter
        def hidden(self, boolValue):
            """
            Control whether this item is hidden in the menu.
            """
            self._hiddenM = boolValue

        @property
        def mealSize(self):
            """
            Get the meal size in small plates.
            """
            return self._mealSizeM

        @property
        def text(self):
            """
            Get the text for an item.

            For headers this is the prompt.
            For items this is the prompt, cost, and postfix.
            """
            if self._headerM:
                return self._promptM
            elif self._costM == 0:
                if self._postM:
                    return "{} {}".format(self._promptM, self._postM)
                else:
                    return self._promptM
            else:
                if self._postM:
                    return "{} ${:0.2f} {}".format(self._promptM, self._costM, self._postM)
                else:
                    return "{} ${:0.2f}".format(self._promptM, self._costM, self._postM)

        @property
        def tooBig(self):
            """
            Is this item too big to consume?
            """
            return self._tooBigM
        
        @tooBig.setter
        def tooBig(self, boolValue):
            """
            Set whether this item is too big to consume.
            """
            self._tooBigM = boolValue

        @property
        def tooGreedy(self):
            """
            Is eating this being too greedy?
            """
            return self._tooGreedyM
        
        @tooGreedy.setter
        def tooGreedy(self, boolValue):
            """
            Set whether consuming this item would be too greedy.
            """
            self._tooGreedyM = boolValue

        @property
        def tooPricey(self):
            """
            Is this item too expensive to buy?
            """
            return self._tooPriceyM
        
        @tooPricey.setter
        def tooPricey(self, boolValue):
            """
            Set whether this item is too small to consume.
            """
            self._tooPriceyM = boolValue

        @property
        def tooSmall(self):
            """
            Is this item too big to consume?
            """
            return self._tooSmallM
        
        @tooSmall.setter
        def tooSmall(self, boolValue):
            """
            Set whether this item is too small to consume.
            """
            self._tooSmallM = boolValue

        @property
        def treat(self):
            """
            Is this item a treat?
            """
            return self._treatM



        # ---------------------------------------------------------------------
        # Accessors
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        def setPost(
                self,
                greedStr="(too greedy)",
                bigStr="(too heavy)",
                smallStr="(too small)",
                priceStr="(too expensive)"
            ):
            """
            Construct the post string according to flags.
            Only one post message is chosen.

            greedStr    - annotation if this would be seen as too greedy
            bigStr      - annotation if the item is too big
            smallStr    - annotation if the item is too small
            priceStr    - annotation if the item is too expensive
            """
            if self._tooBigM:
                self._postM = bigStr
            elif self._tooGreedyM:
                self._postM = greedStr
            elif self._tooSmallM:
                self._postM = smallStr
            elif self._tooPriceyM:
                self._postM = priceStr
            else:
                self._postM = ""


        def __str__(self):
            """
            Generate a string representation for debug purposes.
            """
            if self._headerM:
                return "header:'{}'".format(self._promptM)
            return "item:'{}' ${:0.2f} '{}' avail={} hidden={} msize={}".format(
                    self._promptM, self._costM, self._postM,
                    self.available, self._hiddenM, self._mealSizeM
                )

# Eof
