# Least recently used picker.
#
# Example use:
#   default pickerOpts = []             # Picker options list
#   default picker = Picker(pickerOpts) # Option picker
#   default somewhereLru = []           # Somewhere's LRU list
#
#   somewhere:
#       $ picker.start()
#       $ picker.addOption('quest1')
#       $ picker.addOption('quest2')
#       $ picker.addOption('quest3')
#       # Pick a random option, avoiding the last pick.
#       $ choice = picker.choose(somewhereLru, 1, 2)
#       if choice != None:
#           call expression choice
#
init python:

    # Popo for an option.
    #
    class Option:
        nameM = None
        priorityM = 50
        weightM = 50
        noAddM = False

        # Generate string representation for debug.
        #
        def dbgStr(self):
            return "('{}' pri={} w={} n={})".format(self.nameM, self.priorityM, self.weightM, self.noAddM)



    # Picker class.
    #
    class Picker:
        optionsM = []                       # Options to pick from
        countOptions = 0                    # Original option list size
        countFiltered = 0                   # Options left after filtering for debug
        gnameM = None                       # Default global name

        # Constructor.
        #
        # optList   - list for processing options
        #
        def __init__(self, optList):
            self.optionsM = optList
            self.countOptions = 0
            self.countFiltered = 0
            self.gnameM = None

        # ---------------------------------------------------------------------
        # Methods
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Set the internal list.
        #
        def setList(self, optionsList):
            self.optionsM = optionsList
            if debug:
                dbg("Picker.setList({})".format(type(optionsList)))

        # Clear the list of options to pick from.
        #
        # gname     - default global label name
        #
        def start(self, gname=None):
            self.optionsM.clear()
            self.gnameM= gname
            if debugM:
                dbg("Picker.start('{}'')".format(str(gname)))

        # Add an option that might be chosen.
        #
        # name      - the name of the option that gets returned by choose
        # weight    - the weight of this option, default 50
        # priority  - the priority of this option, defaukt 50
        # noAdd     - do not add to LRU
        #
        def addOption(self, name, weight=50, priority=50, noAdd=False):
            option = Option()
            option.nameM = name
            option.priorityM = priority
            option.weightM = weight
            option.noAddM = noAdd
            self.optionsM.append(option)
            if debugM:
                dbg("Picker.addOption('{}', {}, {}) added {}".format(name, weight, priority, option.dbgStr()))

        # Add a label option that might be chosen.
        #
        # gname     - the global name of the label
        # lname     - the local name of the label beginning with '.'
        # weight    - the weight of this option, default 50
        # priority  - the priority of this option, defaukt 50
        # noAdd     - do not add to LRU
        #
        def addLabel(self, gname, lname=None, weight=50, priority=50, noAdd=False):
            name = gname
            if lname != None:
                name += lname
            self.addOption(name, weight, priority, noAdd)

        # Add a local label option that might be chosen.
        #
        # lname     - the local name of the label beginning with '.'
        # weight    - the weight of this option, default 50
        # priority  - the priority of this option, defaukt 50
        # noAdd     - do not add to LRU
        #
        def addLocal(self, lname, weight=50, priority=50, noAdd=False):
            name = self.gnameM + lname
            self.addOption(name, weight, priority, noAdd)

        # Remove all options.
        #
        def clear(self):
            self.optionsM.clear()


        # Get the count of options added.
        #
        def getCount(self):
            return len(self.optionsM)

        # Get the sum of all the weights of the options in the list.
        #
        def getSumWeights(self):
            result = 0
            for option in self.optionsM:
                result += option.weightM
            return result

        # Check if name is already in the list of options.
        #
        def has(self, name):
            for option in self.optionsM:
                if option.nameM == name:
                    return True
            return False

        # Choose an option, and return the name, or None.
        # The picked option is added to the usedlist and the oldest one dropped.
        # The usedList can have a fixed size with positive values of lruDepth,
        # be allowed to expand to the max needed with lruDepth = 0,
        # or be limited to max options minus the lruDepth with negative values.
        # The options added are consumed by this process.
        #
        # usedList      - the entries recently used
        # lruDepth      - max size of the used list or how many entries not to keep for zero or negative
        # minOptions    - minimum number of things to leave to choose from randomly
        # priorityDiff  - max difference between highest priority option and
        #                 the lowest that will be considered
        #
        def choose(self, usedList=None, lruDepth=3, minOptions=1, priorityDiff=None):
            result = None
            if debug:
                usedTxt = self.usedStr(usedList)
                dbg("Picker.choose(used={}{}{}, {}, {}, {})".format("{size=-4}", usedTxt, "{/size}", lruDepth, minOptions, priorityDiff))
            if debug:
                dbg("picker.choose: options={}{}{}".format("{size=-6}", self.optStr(), "{/size}"))

            # Remove options from the usedList.
            #
            self.countOptions = len(self.optionsM)
            discarded = 0
            if usedList != None and self.countOptions > minOptions:
                for used in usedList:
                    if len(self.optionsM) <= minOptions:
                        break
                    for option in self.optionsM:
                        if option.nameM == used:
                            self.optionsM.remove(option)
                            discarded += 1
                            # if debugM:
                                # dbg("picker.choose: remove recent {}".format(option.dbgStr()))
                            break
            if debugM and discarded > 0:
                dbg("picker.choose: discarded {} that were in usedList leaving {}".format(discarded, len(self.optionsM)))

            # Remove options of low priority compared with highest remaining.
            #
            if priorityDiff != None:
                maxPri = 0
                for option in self.optionsM:
                    maxPri = max(maxPri, option.priorityM)

                minPri = maxPri - priorityDiff
                if debugM:
                    dbg("picker.choose: priority max={} min={}".format(maxPri, minPri))
                trash = []
                discarded = 0
                for option in self.optionsM:
                    if option.priorityM <= minPri:
                        trash.append(option)
                for option in trash:
                    self.optionsM.remove(option)
                    discarded += 1
                    # if debugM:
                        # dbg("picker.choose: discarding {}".format(option.dbgStr()))
                if debugM and discarded > 0:
                    dbg("picker.choose: discarded {} with pri below {} leaving {}".format(discarded, minPri, len(self.optionsM)))

            # Pick one of the remaining options.
            #
            if debug:
                dbg("picker.choose: picking from {}{}{}".format("{size=-4}", self.optStr(), "{/size}"))
            self.countFiltered = len(self.optionsM)
            sumWeights = 0
            for option in self.optionsM:
                sumWeights += option.weightM

            noAdd = False
            if sumWeights > 0:
                rnd = pc.random(0, sumWeights-1)

                if debugM:
                    dbg("picker.choose: sum={} rnd={}".format(sumWeights, rnd))

                for pick in self.optionsM:
                    rnd -= pick.weightM
                    # if debug:
                        # dbg("picker.choose: test name={} rnd={}".format(pick.nameM, rnd))
                    if rnd < 0:
                        result = pick.nameM
                        noAdd = pick.noAddM
                        if debugM:
                            dbg("picker.choose: sum={} name={}".format(rnd, pick.nameM))
                        break

            # Add chosen option to end of used list.
            # If it is already in the list move to start.
            #
            if result != None and usedList != None and not noAdd:
                if result in usedList:
                    usedList.remove(result)

                # How long should LRU list be?
                #
                lruTgt = lruDepth
                if lruDepth <= 0:
                    lruTgt = max(len(usedList), self.countOptions) + lruDepth
                    if debug:
                        dbg("picker.choose: options={} lruLen={} lruTgt={}".format(self.countOptions, len(usedList), lruTgt))

                if len(usedList) >= lruTgt:
                    usedList.pop()
                usedList.insert(0, result)

            # Clear the options.
            #
            self.optionsM.clear()

            if debug:
                dbg("Picker.choose: result={}".format(result))
            return result

        # List contents of opt list as a string.
        # For debug.
        #
        def optStr(self):
            optLen = len(self.optionsM)
            if optLen == 0:
                return "Empty"
            result = "("
            for opt in self.optionsM:
                result += "{}={},{}".format(opt.nameM, opt.weightM, opt.priorityM)
                optLen -= 1
                if optLen > 0:
                    result += ", "
            result += ")"
            return result

        # List contents of usedList to a string.
        # For debug.
        #
        def usedStr(self, usedList):
            if usedList == None:
                return "None"
            usedLen = len(usedList)
            if usedLen == 0:
                return "Empty"
            result = "("
            for used in usedList:
                result += used
                usedLen -= 1
                if usedLen > 0:
                    result += ", "
            result += ")"
            return result
