#
# Unit test support.
#
init python:

    # Unit test reporting class.
    #
    class RpyUnit:
        testsM = 0
        testsFailedM = 0
        testNameM = "None"
        testErrorsM = 0
        showErrorM = False

        # Reset the test results.
        #
        def reset(self):
            self.testsM = 0
            self.testsFailedM = 0
            self.testName = "None"
            self.showErrorM = False

        # Start a test.
        # Increments test count, clears test error count.
        #
        # gname     - name of test (global)
        # lname     - name of test (local - excluding '.') (optional)
        #
        def startTest(self, gname, lname=None):
            if lname != None:
                self.testNameM = gname + "." + lname
            else:
                self.testNameM = gname
            self.testErrorsM = 0
            self.testsM += 1

        # Record a failure.
        # If this is the first error increment testsFailedM
        #
        def fail(self):
            if self.testErrorsM == 0:
                self.testsFailedM += 1
                if not self.showErrorM:
                    dbg("{} failed.".format(self.testNameM))
            self.testErrorsM += 1

        # Report an error.
        # Reports are only shown if showError is set True.
        #
        # expected  - expected value
        # result    - value got
        # txt       - text describing fault (optional)
        #
        def error(self, expected, result, txt=None):
            # dbg("RpyUnit.error({}, {}, {})".format(expected, result, str(txt)))
            self.fail()
            if self.showErrorM:
                if txt == None:
                    msg = "{}: expected {}, got {}".format(self.testNameM, expected, result)
                else:
                    msg = "{}: expected {}, got {}, {}".format(self.testNameM, expected, result, txt)
                dbg(msg)



        # ---------------------------------------------------------------------
        # Accessors.
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Get the report on tests passed out of total.
        #
        def getReport(self):
            msg = "{}/{} tests passed.".format(self.testsM - self.testsFailedM, self.testsM)
            return msg

        # Have all tests passed?
        #
        def isClean(self):
            return self.testsFailedM == 0

        # Has this test passed so far?
        #
        def isTestClean(self):
            return self.testErrorsM == 0

        # Control error reporting.
        #
        # value     - new value True or False
        #
        def setShowError(self, value):
            self.showErrorM = value



    # -------------------------------------------------------------------------
    # Assert methods.
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Assert that two values are equal.
    #
    # expected  - the expected value
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertEquals(expected, value, txt=None):
        if expected != value:
            rpyUnit.error(expected, value, txt)

    # Assert that a value is False.
    #
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertFalse(value, txt=None):
        if value:
            rpyUnit.error(False, value, txt)

    # Assert that a value is None.
    #
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertNone(value, txt=None):
        if value != None:
            rpyUnit.error("None", value, txt)

    # Assert that a value is not None.
    #
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertNotNone(value, txt=None):
        if value == None:
            rpyUnit.error("not None", value, txt)

    # Assert that a value is True.
    #
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertTrue(value, txt=None):
        if not value:
            rpyUnit.error(True, value, txt)

    # Assert that two values are equal within a given tolerance.
    #
    # expected  - the expected value
    # tol       - the tolerance allowed
    # value     - the value
    # txt       - text describing error (optional)
    #
    def assertWithin(expected, tol, value, txt=None):
        if abs(expected - value) > tol:
            minv = expected - tol
            maxv = expected + tol
            expectRange = "{}-{}".format(minv, maxv)
            rpyUnit.error(expectRange, value, txt)

# Unit test control.
#
define  rpyUnit = RpyUnit()

label rpyUnit:

label .report:
    if rpyUnit.testsM == 1:
        if rpyUnit.isClean():
            dbg "Test passed."
        else:
            dbg "Test failed."
    else:
        $ tmp = rpyUnit.getReport()
        dbg "[tmp]"
