#
# Player size changes.
#
init python:

    # Size limits for pc based on mass in kgs.
    # Base on 40kg base weight.
    #
    #       mass kg     weight lb   bfp     notes
    #  0:    40.. 45     88.. 99    < 11.1  A skinny
    #  1:    45.. 50     99..110    < 20.0  B thin
    #  2:    50.. 60    110..132    < 33.3  C normal
    #  3:    60.. 70    132..154    < 42.9  D plump
    #  4:    70.. 80    154..176    < 50.0  E fat
    #  5:    80..108    176..237    < 63.0  F obese
    #  6:   108..136    237..300    < 70.7  G (near sofia)
    #  7:   137..182    300..400    < 78.0  H bbw
    #  8:   182..227    400..500    < 82.4  J
    #  9:   227..273    500..600    < 85.3  K
    # 10:   273..318    600..700    < 87.4  L ssbbw
    # 11:   318..364+   700..800+           M
    # Extended for higher weights (see pc.gsizeMax)
    # 12:      ..369+   800..900            N
    # 13:      ..414+   900..1000+          O
    #
    #           0           1           2           3           4
    sizeLims = (45,         50,         60,         70,         80,
            #   5           6           7           8           9
                108,        300/2.2,    400/2.2,    500/2.2,    600/2.2,
            #   10          11          (12)        (13)        (14)
                700/2.2,    800/2.2,    900/2.2,    1000/2.2,   1100/2.2,
            #   (15)        (16)        (17)        (18)        (19)
                1200/2.2,   1300/2.2,   1400/2.2,   1500/2.2,   1600/2.2,
            #   (20)        (21)        (22)        (23)        (24)
                1700/2.2,   1800/2.2,   1900/2.2,   2000/2.2,   2100/2.2,
            #   (25)        (26)        (27)        (28)        (29)
                2200/2.2,   2300/2.2,   2400/2.2,   2500/2.2,   2600/2.2,
                )



    # -------------------------------------------------------------------------
    # Mass and graphical size
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Look up mass in kg against size limits.
    # Returns size number.
    #
    def sizeLookup(massKg):
        idx = 0;
        for lim in sizeLims:
            if massKg < lim:
                return idx
            else:
                idx += 1

        # Not in table? Extrapolate in 100lb increments.
        #
        idx = 4 + int(math.floor(massKg * 2.2 / 100))
        return idx

    # Lookup maximum mass in kg for a given gsize.
    #
    def kgLookup(gsize):
        if gsize < 0:
            return 40
        if gsize < len(sizeLims):
            return sizeLims[gsize]
        # Extrapolate
        return (gsize - 3) * 100 / 2.2

    # Lookup the mass in kg for a given progress in a gsize range.
    # For example size 7 is 300..400 lbs, so kgAtSizeProgress(7, 75)
    # would return 375lbs/170.45kg.
    #
    # gsize     - gsize range
    # percent   - progress 0..100
    #
    # Returns the total mass in kg.
    #
    def kgAtSizeProgress(gsize, percent):
        lowKg = kgLookup(gsize-1)
        maxKg = kgLookup(gsize)
        deltaKg = maxKg - lowKg
        result = lowKg + deltaKg * percent / 100.0
        return result

    # How far along a size range is a given mass?
    # Result is a percentage 0..100.
    #
    def sizeProgress(massKg, gsize):
        lowKg = kgLookup(gsize-1)
        maxKg = kgLookup(gsize)
        return max(0,min(100,int(round(100*(massKg-lowKg)/(maxKg-lowKg)))))



    # -------------------------------------------------------------------------
    # Body measurements
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Calc the PC waist size from gsize.
    # Gsize can be fractional
    #
    def waistSize(gsize):
        return 2 * int(round(gsize*30/22)) + 28

    # Calc the PC chest size from gsize.
    # Gsize can be fractional
    #
    def chestSize(gsize):
        return 2 * int(round(gsize*22/22)) + 32

    # Calc the PC overbust size from gsize.
    # Gsize can be fractional
    #
    def overBust(gsize):
        return 2 * int(round(gsize*33/22)) + 32

    # Calc the PC hip size (circumference) from gsize.
    # Gsize can be fractional
    #
    def hipSize(gsize):
        base = round(gsize*34/11)
        extra = max(0, round(gsize*54/11) - 27)
        result = 2 * int(round((base + extra)/2)) + 34
        if debugM:
            dbg("hipSize({})={} base={} extra={}".format(gsize, result, base, extra))
        return result

    # Calculate hips width in inches from hip circumference.
    #
    def hipsWidthIn(hipCirc):
        thighDia = hipCirc / 5.14
        result = int(round(2.0 * thighDia))
        if debugM:
            dbg("hipsWidthIn({})={}".format(hipCirc, result))
        return result

    # Calculate hips spread (sat) in inches from hips width.
    #
    def hipsSpreadIn(hipsWidth):
        result = int(round(1.25 * hipsWidth))
        if debugM:
            dbg("hipsSpreadIn({})={}".format(hipsWidth, result))
        return result

    # Calculate hip circumference of pc.
    # Uses sizes > 11
    #
    def pcHipsCircIn():
        pcsize = sizeLookup(pc.getMass())
        asize = pcsize + sizeProgress(pc.getMass(), pcsize)/100.0
        hipCirc = hipSize(asize)
        if debugM:
            dbg("pcHipsCircIn()={} pcsize={} asize={:0.2f}".format(hipCirc, pcsize, asize))
        return hipCirc

    # Calculate standing width of pc.
    # Uses sizes > 11
    #
    def pcHipsWidthIn():
        pcsize = sizeLookup(pc.getMass())
        asize = pcsize + sizeProgress(pc.getMass(), pcsize)/100.0
        hipCirc = hipSize(asize)
        hipsWidth = hipsWidthIn(hipCirc)
        if debugM:
            dbg("pcHipsWidthIn()={} pcsize={} asize={:0.2f} hipCirc={}".format(hipsWidth, pcsize, asize, hipCirc))
        return hipsWidth

    # Calculate seated width of pc.
    # Uses sizes > 11
    #
    def pcHipsSpreadIn():
        pcsize = sizeLookup(pc.getMass())
        asize = pcsize + sizeProgress(pc.getMass(), pcsize)/100.0
        hipCirc = hipSize(asize)
        hipsWidth = hipsWidthIn(hipCirc)
        hipsSpread = hipsSpreadIn(hipsWidth)
        if debugM:
            dbg("pcHipsSpreadIn()={} pcsize={} asize={:0.2f} hipCirc={} hipsWidth={}".format(hipsSpread, pcsize, asize, hipCirc, hipsWidth))
        return hipsSpread



    # -------------------------------------------------------------------------
    # RenPy calls
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Check for a size change and call the appropriate size up or down.
    #
label pcSize:
    # $ dbgLog("pcSize")
    $ gsize = pc.getSize()
    if flags.isConsume('sizeU'):
        call .sizeUp from pc_size_up
    elif flags.isConsume('sizeD'):
        call .sizeDn from pc_size_down
    return



    # Check and flag size changes.
    # Updates pc.gsize
    # Limited to sizes 0 .. pc.gsizeMax
    #
label .check:
    $ nsize = min(pc.gsizeMax, sizeLookup(pc.getMass()))
    $ osize = pc.gsize
    if nsize == osize:
        if debug:
            dbg "check: size unchanged"
        return

    $ pc.gsize = nsize

    if nsize > osize:
        $ flags.set("sizeU")
        $ flags.clear("sizeD")
        if debug:
            dbg "check: size increase to [nsize]"
    else:
        $ flags.set("sizeD")
        $ flags.clear("sizeU")
        if debug:
            dbg "check: size decrease to [nsize]"
    return



    # Player size stage decrease.
    #
    # Call descriptive label and set/clear flags.
    #
label .sizeDn:
    $ dbgLog("pcSize.sizeDn")
    $ renpy.dynamic('renLabel', 'unsure')
    $ unsure = pc.isDietConsider()

    # Try <location>Size.sizeDn<gsize>
    #
    $ renLabel = "{}Size.sizeDn{}".format(world.getLoc(), gsize)
    if world.hasLabel(renLabel):
        call expression renLabel from pc_size_dn_dyn
    else:
        dbg "In pcSize.sizeDn: selected label \"[renLabel]\" was not found."

    # Set size down flags.
    #
    $ flags.setEach('joeSizeD', 'lacySizeD', 'nicolaSizeD', 'regSizeD', 'sofiaSizeD', 'steveSizeD')
    if world.isPartWork() and world.getPartWorkLabel().startswith('nwVachonWarehouse'):
        $ flags.set('wareSizeD')

    # Clear size up flags.
    #
    $ flags.clearEach('joeSizeU', 'lacySizeU', 'nicolaSizeU', 'regSizeU', 'sofiaSizeU', 'steveSizeU', 'wareSizeU')
    return



    # Player size stage increase.
    #
    # Call descriptive label and set/clear flags.
    #
label .sizeUp:
    $ dbgLog("pcSize.sizeUp")
    $ renpy.dynamic('renLabel', 'unsure')
    $ unsure = pc.isDietConsider()

    # Try <location>Size.sizeUp<gsize>
    #
    $ renLabel = "{}Size.sizeUp{}".format(world.getLoc(), gsize)
    if world.hasLabel(renLabel):
        call expression renLabel from pc_size_up_dyn
    else:
        dbg "In pcSize.sizeUp: selected label \"[renLabel]\" was not found."

    # Think about dieting.
    #
    if pc.isDietConsider() and not pc.isDiet():
        $ flags.set('pcThinkDiet')

    # Set size up flags.
    #
    $ flags.setEach('joeSizeU', 'lacySizeU', 'nicolaSizeU', 'regSizeU', 'sofiaSizeU', 'steveSizeU')
    if world.isPartWork() and world.getPartWorkLabel().startswith('nwVachonWarehouse'):
        $ flags.set('wareSizeU')

    # Clear size down flags.
    #
    $ flags.clearEach('joeSizeD', 'lacySizeD', 'nicolaSizeD', 'regSizeD', 'sofiaSizeD', 'steveSizeD', 'wareSizeD')
    return
