#
# Useful functions for numbers.
#
init python:
    # Cardinal number words 0..19.
    #
    cardinals = [
        'zero',
        'one',
        'two',
        'three',
        'four',
        'five',
        'six',
        'seven',
        'eight',
        'nine',
        'ten',
        'eleven',
        'twelve',
        'thirteen',
        'fourteen',
        'fifteen',
        'sixteen',
        'seventeen',
        'eighteen',
        'nineteen',
    ]



    # Cardinal decades.
    #
    cardDec = [
        'zero',
        'ten',
        'twenty',
        'thirty',
        'forty',
        'fifty',
        'sixty',
        'seventy',
        'eighty',
        'ninety',
    ]



    # Get the cardinal for a value: one, two, three ...  nine hundred and
    # ninety nine million nine hundred and ninety nine thousand nine hundred
    # and ninety nine.
    #
    # valInt    - integer to convert
    # numMin    - number where numeric conversion is used
    # prefHun   - True to prefer using hundreds over thousands (eg. eleven hundred)
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardinal(valInt, numMin=None, prefHun=False, prefA=False):
        if numMin == None:
            numMin = 1000000000
        # Simple case 0..19
        if valInt < len(cardinals):
            return cardinals[valInt]
        # Exceeds use numerics value
        if valInt >= numMin:
            return "{:0,}".format(valInt)
        # Value must now be 20..numMin
        if valInt < 10000 and prefHun and valInt // 100 != 10:
            return cardHundNone(valInt, prefA)

        # Build up number in millioms, thousands ...
        result = None
        hunInt = valInt % 1000
        thouInt = valInt // 1000 % 1000
        milInt = valInt // 1000000
        thouStr = None
        milStr = None

        if milInt >= 1:
            milStr = cardHundNone(milInt, prefA)
            prefA = False
        if thouInt >= 1:
            thouStr = cardHundNone(thouInt, prefA)
            prefA = False
        hunStr = cardHundNone(hunInt, prefA)

        if thouInt == 0 and milInt == 0:
            result = hunStr
        else:
            parts = []
            if thouStr != None:
                parts.insert(0, thouStr + " thousand")
            if milStr != None:
                parts.insert(0, milStr + " million")
            result = " ".join(parts)
            if hunStr != None:
                if 'and' in hunStr:
                    result += " " + hunStr
                else:
                    result += " and " + hunStr
        return result



    # Get the cardinal for a value rounded up to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round down to
    # numMin    - number where numeric conversion is used
    # prefHun   - True to prefer using hundreds over thousands (eg. eleven hundred)
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardCeil(valIntFl, modInt, numMin=None, prefHun=False, prefA=False):
        valInt = ceilPrec(valIntFl, modInt)
        return cardinal(valInt, numMin, prefHun, prefA)



    # Get the cardinal for a value rounded down to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round down to
    # numMin    - number where numeric conversion is used
    # prefHun   - True to prefer using hundreds over thousands (eg. eleven hundred)
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardFloor(valIntFl, modInt, numMin=None, prefHun=False, prefA=False):
        valInt = floorPrec(valIntFl, modInt)
        return cardinal(valInt, numMin, prefHun, prefA)



    # Get the cardinal for a value rounded to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round to
    # numMin    - number where numeric conversion is used
    # prefHun   - True to prefer using hundreds over thousands (eg. eleven hundred)
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardRound(valIntFl, modInt, numMin=None, prefHun=False, prefA=False):
        valInt = roundPrec(valIntFl, modInt)
        return cardinal(valInt, numMin, prefHun, prefA)



    # Get the nearest cardinal in decades, rounded down.
    # Works for value 10 .. 999,999,999.
    #
    # valIntFl  - integer/float to convert
    # numMin    - number where numeric conversion is used
    # prefHun   - True to prefer using hundreds over thousands (eg. eleven hundred)
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardDecades(valIntFl, numMin=None, prefHun=False, prefA=True):
        return cardFloor(valIntFl, 10, numMin, prefHun, prefA)



    # Get the cardinal for values 1..99 inclusive.
    # Input of zero returns None.
    #
    # valInt    - value to convert
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardTensNone(valInt, prefA=False):
        valInt %= 100
        result = None
        if valInt > 0:
            if valInt == 1 and prefA:
                result = "a"
            elif valInt < len(cardinals):
                result = cardinals[valInt]
            else:
                decade = valInt // 10
                units = valInt % 10
                if units == 0:
                    result = cardDec[decade]
                else:
                    result = cardDec[decade] + " " + cardinals[units]
        # dbg("cardTensNone({}, {})={}".format(valInt, prefA, result))
        return result



    # Get the cardinal for values 1..9999 inclusive.
    # Input of zero returns None.
    #
    # valInt    - value to convert
    # prefA     - prefer "a" to "one" in first digit
    #
    def cardHundNone(valInt, prefA=False):
        valInt %= 10000
        result = None
        if valInt > 0:
            if valInt < len(cardinals):
                result = cardinals[valInt]
            else:
                hunStr = cardTensNone(valInt // 100, prefA)
                decStr = cardTensNone(valInt % 100)
                if hunStr == None:
                    result = decStr
                elif decStr == None:
                    result = hunStr + " hundred"
                else:
                    result = hunStr + " hundred and " + decStr
        # dbg("cardHundNone({}, {})={}".format(valInt, prefA, result))
        return result



    # Round a float/int up to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round up to
    #
    def ceilPrec(valIntFl, modInt):
        return int(math.ceil(float(valIntFl) / modInt)) * modInt



    # Round a float/int down to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round down to
    #
    def floorPrec(valIntFl, modInt):
        return int(math.floor(float(valIntFl) / modInt)) * modInt



    # Round a float/int to a whole number of modInt.
    #
    # valIntFl  - integer/float to convert
    # modInt    - number to round to
    #
    def roundPrec(valIntFl, modInt):
        return int(round(float(valIntFl) / modInt)) * modInt



    # Ordinal number words.
    #
    ordinals = [
        'first',
        'second',
        'third',
        'fourth',
        'fifth',
        'sixth',
        'seventh',
        'eighth',
        'nineth',
        'tenth',
        'eleventh',
        'twelfth',
        'thirteenth',
        'fourteenth',
        'fifteenth',
        'sixteenth',
        'seventeenth',
        'eighteenth',
        'nineteenth',
        'twentieth'
    ]



    # Get the ordinal for a value: first, second, third ... 101st, 102nd, 103rd, 104th
    #
    def ordinal(valInt):
        if valInt == 0:
            return '0th'
        idx = valInt - 1
        if idx < 0:
            return '-ve'
        if idx < len(ordinals):
            return ordinals[idx]
        idx = valInt % 10
        if idx == 1:
            return str(valInt)+'st'
        if idx == 2:
            return str(valInt)+'nd'
        if idx == 3:
            return str(valInt)+'rd'
        return str(valInt)+'th'



    # Create a list of cardinal values evenly spaced including the start and end values.
    # Uses a classic integer digital differential analyser.
    # So cardList(2, 11, 5) returns ["two", "five", "seven", "nine", "eleven"].
    # Useful for listing plates, tubs and so on during gluttonous sessions.
    #
    # startNum  - ordinal to start with (inclusive)
    # endNum    - ordinal to end with (inclusive)
    # entries   - number of entries
    #
    def cardList(startNum, endNum, entries):
        result = []
        nums = numList(startNum, endNum, entries)
        for num in nums:
            result.append(cardinal(num))
        return result



    # Create a list of numbers evenly spaced including the start and end values.
    # Uses a classic integer digital differential analyser.
    # So numList(2, 11, 5) returns [2, 5, 7, 9, 11].
    # Useful for listing plates, tubs and so on during gluttonous sessions.
    #
    # startNum  - number to start with (inclusive)
    # endNum    - number to end with (inclusive)
    # entries   - number of entries
    #
    def numList(startNum, endNum, entries):
        result = []
        delta = endNum - startNum
        acc = (entries + 1) // 2
        num = startNum
        for step in range(0, entries):
            result.append(num)
            acc += delta
            num += acc // (entries - 1)
            acc %= (entries - 1)
        return result



    # Create a list of ordinal values evenly spaced including the start and end values.
    # Uses a classic integer digital differential analyser.
    # So ordList(2, 11, 5) returns ["second", "fifth", "seventh", "nineth", "eleventh"].
    # Useful for listing plates, tubs and so on during gluttonous sessions.
    #
    # startNum  - ordinal to start with (inclusive)
    # endNum    - ordinal to end with (inclusive)
    # entries   - number of entries
    #
    def ordList(startNum, endNum, entries):
        result = []
        nums = numList(startNum, endNum, entries)
        for num in nums:
            result.append(ordinal(num))
        return result



    # Is a value between limit - range (inclusive) and limit (exclusive)?
    #
    def isInRange(value, limit, range):
        if value >= limit:
            return False
        if value < limit - range:
            return False
        return True

